package com.ruoyi.postCheck.controller;

import com.ruoyi.common.constant.PostCheckedReportEnum;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.postCheck.domain.PostCheckedReport;
import com.ruoyi.postCheck.domain.history.CheckStatisticsView;
import com.ruoyi.postCheck.domain.problemStatistics.CheckedProblemDeptDetaailView;
import com.ruoyi.postCheck.model.DeptDetaailViewModel;
import com.ruoyi.postCheck.service.IPostCheckedReportService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysPostService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 岗检报告管理Controller
 *
 * @author csg
 * @date 2023-12-28
 */
@Api(tags = "6.1.1-岗检历史")
@RestController
@RequestMapping("/postCheck/history")
public class PostCheckedHistoryController extends BaseController {
    @Autowired
    private IPostCheckedReportService postCheckedReportService;

    @Autowired
    private ISysDeptService sysDeptService;

    @Autowired
    private ISysPostService sysPostService;


    @ApiOperation("1、岗检历史列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键ID", dataType = "Long", dataTypeClass = Long.class),
            @ApiImplicitParam(name = "reportNo", value = "编码", dataType = "String", dataTypeClass = String.class),
            @ApiImplicitParam(name = "reportName", value = "岗检报告名称", dataType = "String", dataTypeClass = String.class),
//            @ApiImplicitParam(name = "generateTime", value = "生成时间", dataType = "Date", dataTypeClass = Date.class),
            @ApiImplicitParam(name = "pageNum", value = "页码从1开始", dataType = "Integer", dataTypeClass = Integer.class),
            @ApiImplicitParam(name = "pageSize", value = "分页大小", dataType = "Integer", dataTypeClass = Integer.class)
    })
    @GetMapping("/list")
    public TableDataInfo list(PostCheckedReport postCheckedReport) {
        startPage("desc", "t1.create_time");
        postCheckedReport.setStatus(PostCheckedReportEnum.STATUS_PASS.getValue());
        List<PostCheckedReport> list = postCheckedReportService.selectPostCheckedReportList(postCheckedReport);
        return getDataTable(list);
    }


    @ApiOperation("2、查看-第一个表格：1-检查部门；2-岗检岗位；3-完整性问题；4-有效性问题；5-适宜性问题；6-问题总数；7-已整改问题；")
    @GetMapping("/checkStatistics")
    public AjaxResult checkStatistics() {

//        第一步：、1-检查部门；2-岗检岗位；
//        select * from "public"."post_plan";
//        select * from "public"."post_plan_info";
//        第二步、3-完整性问题；4-有效性问题；5-适宜性问题；6-问题总数；7-已整改问题；
//        select * from "public"."post_checked_problem";

        List<CheckStatisticsView> list = new ArrayList<>();

        CheckStatisticsView data1 = new CheckStatisticsView();
        data1.setValue(1);
        data1.setPostNum(11);
        list.add(data1);

        CheckStatisticsView data2 = new CheckStatisticsView();
        data2.setValue(2);
        data2.setPostNum(22);
        list.add(data2);

        CheckStatisticsView data3 = new CheckStatisticsView();
        data3.setValue(3);
        data3.setPostNum(33);
        list.add(data3);

        CheckStatisticsView data4 = new CheckStatisticsView();
        data4.setValue(4);
        data4.setPostNum(44);
        list.add(data4);

        CheckStatisticsView data5 = new CheckStatisticsView();
        data5.setValue(5);
        data5.setPostNum(55);
        list.add(data5);

        CheckStatisticsView data6 = new CheckStatisticsView();
        data6.setValue(6);
        data6.setPostNum(66);
        list.add(data6);


        CheckStatisticsView data7 = new CheckStatisticsView();
        data7.setValue(7);
        data7.setPostNum(77);
        list.add(data7);
        return success(postCheckedReportService.checkStatistics());
    }

    @ApiOperation("3、查看-第二个表格：岗检问题")
    @GetMapping("/deptDetaailView")
    public AjaxResult deptDetaailView(DeptDetaailViewModel model) { //notParam不是参数-是返回信息-方便接口联调

        //TODO 原型上Table第三列列是“岗位”，和产品沟通一下，应该无法匹配到这个信息，因为问题录入时没有设置岗位信息，是否考虑把这列删除掉
//        查询问题表：按duty_type（职责类别：1-业务职责；2-党建及党风廉政；3-安全环保职责；4-合规保密职；）分组统计问题属性的数量（problem_attribute问题属性（1-完整问题；2-有效性问题；3-适宜性问题；））
//        select * from "public"."post_checked_problem";

        List<CheckedProblemDeptDetaailView> list = new ArrayList<>();

        //TODO 第一步：得到各个部门ID、部门名称、统计时间
        //TODO 第二步：得到4个职责类别的问题数量及整改完成数量(1-业务职责；2-党建及党风廉政；3-安全环保职责；4-合规保密职；)
        //TODO 第三步：设置部门的问题属性（1-完整问题；2-有效性问题；3-适宜性问题；）数量：遍历按职责类别计算问题属性数量
        CheckedProblemDeptDetaailView data1 = new CheckedProblemDeptDetaailView();
        data1.setDeptId(1l); //部门ID
        data1.setDeptName("测试部门名称1");// 部门名称
//        data1.setPostId(100l); //岗位ID
//        data1.setPostName("岗位名称1");// 岗位名称
        data1.setStatisticsTime(new Date());// 统计时间
        CheckedProblemDeptDetaailView.ProblemAttributeInner child1 = new CheckedProblemDeptDetaailView.ProblemAttributeInner();
        child1.setProblemComplete(1); // 完整性数量
        child1.setProblemSuitable(2); // 适宜性数量
        child1.setProblemEffective(3);// 有效性数量
        data1.getChildren().add(child1);
        list.add(data1);

        CheckedProblemDeptDetaailView data2 = new CheckedProblemDeptDetaailView();
        data2.setDeptId(2l); //部门ID
        data2.setDeptName("测试部门名称2");// 部门名称
//        data2.setPostId(100l); //岗位ID
//        data2.setPostName("岗位名称1");// 岗位名称
        data2.setStatisticsTime(new Date());// 统计时间
        CheckedProblemDeptDetaailView.ProblemAttributeInner child2 = new CheckedProblemDeptDetaailView.ProblemAttributeInner();
        child1.setProblemComplete(2); // 完整性数量
        child1.setProblemSuitable(3); // 适宜性数量
        child1.setProblemEffective(4);// 有效性数量
        data2.getChildren().add(child2);
        list.add(data2);

        return success(postCheckedReportService.deptDetaailView(model));
    }

}
