package com.ruoyi.postCheck.service.impl;

import java.util.*;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.conditions.query.QueryChainWrapper;
import com.ruoyi.common.constant.PostListEnum;
import com.ruoyi.common.enums.DeptCode;
import com.ruoyi.common.exception.base.BaseException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.postCheck.domain.PostCheckedItems;
import com.ruoyi.postCheck.mapper.PostListInfoMapper;
import com.ruoyi.postCheck.service.IPostListInfoService;
import com.ruoyi.system.service.ISysPostService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.stream.Collectors;

import com.ruoyi.common.utils.StringUtils;
import org.springframework.transaction.annotation.Transactional;
import com.ruoyi.postCheck.domain.PostListInfo;
import com.ruoyi.postCheck.mapper.PostListMapper;
import com.ruoyi.postCheck.domain.PostList;
import com.ruoyi.postCheck.service.IPostListService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.util.CollectionUtils;

/**
 * 岗检责任制清单Service业务层处理
 *
 * @author csg
 * @date 2023-12-20
 */
@Service
public class PostListServiceImpl extends ServiceImpl<PostListMapper, PostList> implements IPostListService {
    @Autowired
    private PostListMapper postListMapper;
    @Autowired
    private PostListInfoMapper postListInfoMapper;

    @Autowired
    private IPostListInfoService postListInfoService;

    @Autowired
    private ISysPostService sysPostService;

    /**
     * 查询岗检责任制清单
     *
     * @param id 岗检责任制清单主键
     * @return 岗检责任制清单
     */
    @Override
    public PostList selectPostListById(Long id) {
        PostList data = postListMapper.selectPostListById(id);
        if (data != null) {
            Long dataId = data.getId();
            PostListInfo info = new PostListInfo();
            info.setPostListId(dataId);
            List<PostListInfo> infoList = postListInfoService.selectPostListInfoList(info);
            data.setChildren(infoList);
        }
        return data;
    }

    /**
     * 查询岗检责任制清单列表
     *
     * @param postList 岗检责任制清单
     * @return 岗检责任制清单
     */
    @Override
    public List<PostList> selectPostListList(PostList postList) {
        List<PostList> postLists = postListMapper.selectPostListList(postList);

        //查询详情
        if (!CollectionUtils.isEmpty(postLists)) {
            List<Long> postListIds = postLists.stream().map(PostList::getId).collect(Collectors.toList());
            LambdaQueryWrapper<PostListInfo> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.in(PostListInfo::getPostListId, postListIds);
            queryWrapper.orderByAsc(PostListInfo::getId);
            List<PostListInfo> postListInfoList = postListInfoService.list(queryWrapper);
            //详情分组
            Map<Long, List<PostListInfo>> postListInfoListGroupMap = postListInfoList.stream().collect(Collectors.groupingBy(PostListInfo::getPostListId));
            //设置详情
            for (PostList data : postLists) {
                data.setChildren(postListInfoListGroupMap.get(data.getId()));
            }
        }

        return postLists;
    }

    /**
     * 新增岗检责任制清单
     *
     * @param postList 岗检责任制清单
     * @return 结果
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int insertPostList(PostList postList) {
        //TODO 自增主键修改为-MyBatis-Plus雪花算法(截取5位解决VUE精度丢失BUG)-系统登录日志
//        if (postList.getId() == null)
        postList.setId(Long.parseLong(IdWorker.getIdStr().substring(4)));
        postList.setCreateDefault();
        postList.setCreationDate(new Date());//编制时间
        postList.setRevisionDate(new Date());//修订时间
        postList.setStatus(PostListEnum.STATUS_WAIT.getValue());//待审核
        int rows = postListMapper.insertPostList(postList);

        //新增责任制清单明细
        insertPostListInfo(postList);

        //TODO 更新业务职责的状态=待审核：只要部门修改数据，人力资源都需要重新审核单据
        //1、按区域类型+主表id批量更新明细表状态、审核人、审核时间、未通过理由
        LambdaUpdateWrapper<PostListInfo> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        //条件
        lambdaUpdateWrapper.eq(PostListInfo::getPostListId, postList.getId());
        lambdaUpdateWrapper.eq(PostListInfo::getType, PostListEnum.TYPE_ONE.getValue());
        //参数
        lambdaUpdateWrapper.set(PostListInfo::getStatus, PostListEnum.STATUS_WAIT.getValue());
        lambdaUpdateWrapper.set(PostListInfo::getStatusTime, null);
        lambdaUpdateWrapper.set(PostListInfo::getStatusByUserId, null);
        lambdaUpdateWrapper.set(PostListInfo::getStatusByUserName, null);
        lambdaUpdateWrapper.set(PostListInfo::getReason, null);
        postListInfoMapper.update(null, lambdaUpdateWrapper);

        return rows;
    }

    /**
     * 修改岗检责任制清单
     *
     * @param postList 岗检责任制清单
     * @return 结果
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int updatePostList(PostList postList) {
        postList.setUpdateDefault();
        //TODO 待改造，不可以将全部删除完，哪个部门修改了删除那个部门的，其他部门的不能删除。
        //删除明细
//        postListMapper.deletePostListInfoByPostListId(postList.getId());
        //删除明细：按类型只删除本次包含的主要职责
        List<PostListInfo> children = postList.getChildren();
        Map<Integer, List<PostListInfo>> typeMapList = children.stream().collect(Collectors.groupingBy(PostListInfo::getType));
        Set<Integer> typeList = typeMapList.keySet();
        if (CollectionUtils.isEmpty(typeList)) {
            throw new BaseException("请至少设置一项“主要职责”！");
        }
        LambdaQueryWrapper<PostListInfo> deleteWrapper = new LambdaQueryWrapper<>();
        deleteWrapper.eq(PostListInfo::getPostListId, postList.getId());
        deleteWrapper.in(PostListInfo::getType, typeList);
        postListInfoMapper.delete(deleteWrapper);

        //重新添加
        insertPostListInfo(postList);

        postList.setStatus(PostListEnum.STATUS_WAIT.getValue());
        int updateCount = postListMapper.updatePostList(postList);

        //TODO 更新业务职责的状态=待审核：只要部门修改数据，人力资源都需要重新审核单据
        //1、按区域类型+主表id批量更新明细表状态、审核人、审核时间、未通过理由
        LambdaUpdateWrapper<PostListInfo> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        //条件
        lambdaUpdateWrapper.eq(PostListInfo::getPostListId, postList.getId());
        lambdaUpdateWrapper.eq(PostListInfo::getType, PostListEnum.TYPE_ONE.getValue());
        //参数
        lambdaUpdateWrapper.set(PostListInfo::getStatus, PostListEnum.STATUS_WAIT.getValue());
        lambdaUpdateWrapper.set(PostListInfo::getStatusTime, null);
        lambdaUpdateWrapper.set(PostListInfo::getStatusByUserId, null);
        lambdaUpdateWrapper.set(PostListInfo::getStatusByUserName, null);
        lambdaUpdateWrapper.set(PostListInfo::getReason, null);
        postListInfoMapper.update(null, lambdaUpdateWrapper);

        return updateCount;
    }

    /**
     * 批量删除岗检责任制清单
     *
     * @param ids 需要删除的岗检责任制清单主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deletePostListByIds(Long[] ids) {
        postListMapper.deletePostListInfoByPostListIds(ids);
        return postListMapper.deletePostListByIds(ids);
    }

    /**
     * 删除岗检责任制清单信息
     *
     * @param id 岗检责任制清单主键
     * @return 结果
     */
    @Transactional
    @Override
    public int deletePostListById(Long id) {
        postListMapper.deletePostListInfoByPostListId(id);
        return postListMapper.deletePostListById(id);
    }

    /**
     * 新增岗检责任制清单明细信息
     *
     * @param postList 岗检责任制清单对象
     */
    public void insertPostListInfo(PostList postList) {
        List<PostListInfo> children = postList.getChildren();
        Long id = postList.getId();
        if (!CollectionUtils.isEmpty(children)) {
            List<PostListInfo> list = new ArrayList<>();
            for (PostListInfo postListInfo : children) {
                Integer type = postListInfo.getType();
                //TODO 自增主键修改为-MyBatis-Plus雪花算法(截取5位解决VUE精度丢失BUG)-系统登录日志
//                if (postListInfo.getId() == null)
                postListInfo.setId(Long.parseLong(IdWorker.getIdStr().substring(4)));
                postListInfo.setPostListId(id);

                postListInfo.setCreateDefault();
                postListInfo.setCreateUserId(SecurityUtils.getUserId());//创建者ID
                //自动审核：任职资格与能力素质要求
                if (type == PostListEnum.TYPE_FIVE.getValue()) {
                    postListInfo.setStatus(PostListEnum.STATUS_PASS.getValue());//状态，可能需要重新设计，五大部门按类型区分状态
                    postListInfo.setStatusByUserId(SecurityUtils.getUserId());
                    postListInfo.setStatusByUserName(SecurityUtils.getLoginUser().getUser().getNickName());
                    postListInfo.setReason("Auto");
                }
                //其他都是待审核
                else {
                    postListInfo.setStatus(PostListEnum.STATUS_WAIT.getValue());//状态，可能需要重新设计，五大部门按类型区分状态
                }

                list.add(postListInfo);
                //非业务职责修改时将业务职责设置为待审核
//                if (postListInfo.getType() != PostListEnum.TYPE_ONE.getValue()) {
//
//                }
            }
            if (list.size() > 0) {
                postListMapper.batchPostListInfo(list);
            }
        }
    }


    /**
     * @param deptId 部门ID
     * @param postId 岗位ID
     * @param type   检查项目（业务职责、党建党风、安全环保……）
     * @return
     */
    @Override
    public List<PostCheckedItems> contentListByType(Long deptId, Long postId, Integer type) {

        return postListMapper.contentListByType(deptId, postId, type);
    }

    /**
     * 岗位责任制清单审核
     *
     * @param type   审核的类型（1-业务职责；2-党建及党风廉政建设职责；3-安全环保职责；4-合规保密职；5-任职资格与能力素质要求）
     * @param id     主键ID
     * @param status 状态：3-通过；4-驳回；
     * @param reason 未通过理由（驳回时必选）
     * @return
     */
    @Override
    public int approve(Integer type, Long id, Integer status, String reason) {
        if (type == null || id == null || status == null) {
            throw new BaseException("参数不全，请检查！");
        }
        if (status == PostListEnum.STATUS_BACK.getValue() && StringUtils.isBlank(reason)) {
            throw new BaseException("请填写未通过理由！");
        }

        if (status == PostListEnum.STATUS_PASS.getValue() && status == PostListEnum.STATUS_BACK.getValue()) {
            throw new BaseException("状态错误，请检查！");
        }

        //1、判断权限
        //1-业务职责（人力资源部）
        if (type == PostListEnum.TYPE_ONE.getValue()) {
            List<Long> userHaveAuthorityList = sysPostService.userHaveAuthorityList(DeptCode.DEPT_HR.getCode());
            if (!userHaveAuthorityList.contains(SecurityUtils.getUserId())) {
                throw new BaseException("当前账号无权限，请检查！【" + type + "】");
            }

        }
        //2-党建及党风廉政建设职责（党建工作部）
        else if (type == PostListEnum.TYPE_TWO.getValue()) {
            List<Long> userHaveAuthorityList = sysPostService.userHaveAuthorityList(DeptCode.DEPT_PARTY.getCode());
            if (!userHaveAuthorityList.contains(SecurityUtils.getUserId())) {
                throw new BaseException("当前账号无权限，请检查！【" + type + "】");
            }
        }
        //3-安全环保职责（健康安全环保中心）
        else if (type == PostListEnum.TYPE_THREE.getValue()) {
            List<Long> userHaveAuthorityList = sysPostService.userHaveAuthorityList(DeptCode.DEPT_HSE.getCode());
            if (!userHaveAuthorityList.contains(SecurityUtils.getUserId())) {
                throw new BaseException("当前账号无权限，请检查！【" + type + "】");
            }
        }
        //4-合规保密职责（内控法务部&办公室）
        else if (type == PostListEnum.TYPE_FOUR.getValue()) {
            List<Long> userHaveAuthorityList = sysPostService.userHaveAuthorityList(DeptCode.DEPT_ICFA.getCode());
            List<Long> userHaveAuthorityListV2 = sysPostService.userHaveAuthorityList(DeptCode.DEPT_OFFICE.getCode());
            userHaveAuthorityList.addAll(userHaveAuthorityListV2);

            if (!userHaveAuthorityList.contains(SecurityUtils.getUserId())) {
                throw new BaseException("当前账号无权限，请检查！【" + type + "】");
            }
        }
        //不需要审核，新增修改时自动审核的
        //5-任职资格与能力素质要求（不需要审核）
//        else if (type == PostListEnum.TYPE_ONE.getValue()) {
//
//        }
        else {
            throw new BaseException("类型错误，请检查！");
        }


        //2、判断人力资源最后审核
        if (type == PostListEnum.TYPE_ONE.getValue()) {
            //查询其他类型待审核的记录
            LambdaQueryWrapper<PostListInfo> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(PostListInfo::getPostListId, id);
            queryWrapper.ne(PostListInfo::getType, PostListEnum.TYPE_ONE.getValue());//类型（1-业务职责；2-党建及党风廉政建设职责；3-安全环保职责；4-合规保密职；5-任职资格与能力素质要求）
            queryWrapper.ne(PostListInfo::getStatus, PostListEnum.STATUS_WAIT.getValue());//分区域审核-状态：1-待审核；3-通过；4-驳回；

            List<PostListInfo> postListInfoList = postListInfoMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(postListInfoList)) {
                Set<Integer> typeSet = postListInfoList.stream().map(t -> t.getType()).collect(Collectors.toSet());
                String typeView = "";
                for (Integer typeIng : typeSet) {
                    typeView += PostListEnum.getTypeView(typeIng) + "；";
                }
                throw new BaseException("以下部门还未审核，请检查！" + typeView);
            }
        }


        //1、按区域类型+主表id批量更新明细表状态、审核人、审核时间、未通过理由
        LambdaUpdateWrapper<PostListInfo> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        //条件
        lambdaUpdateWrapper.eq(PostListInfo::getPostListId, id);
        lambdaUpdateWrapper.eq(PostListInfo::getType, type);
        //参数
        lambdaUpdateWrapper.set(PostListInfo::getStatus, status);
        lambdaUpdateWrapper.set(PostListInfo::getStatusTime, new Date());

        //通过-参数
        if (status == PostListEnum.STATUS_PASS.getValue()) {
            lambdaUpdateWrapper.set(PostListInfo::getStatusByUserId, SecurityUtils.getUserId());
            lambdaUpdateWrapper.set(PostListInfo::getStatusByUserName, SecurityUtils.getLoginUser().getUser().getNickName());
            lambdaUpdateWrapper.set(PostListInfo::getReason, null);
        }
        //驳回-参数
        else if (status == PostListEnum.STATUS_BACK.getValue()) {
            lambdaUpdateWrapper.set(PostListInfo::getStatusByUserId, null);
            lambdaUpdateWrapper.set(PostListInfo::getStatusByUserName, null);
            lambdaUpdateWrapper.set(PostListInfo::getReason, reason);
        }
        int updateCount = postListInfoMapper.update(null, lambdaUpdateWrapper);
        if (updateCount == 0) {
            throw new BaseException("更新状态失败，请稍后重试！");
        }

        return updateCount;
    }
}
