package com.ruoyi.postCheck.service.impl;

import java.util.List;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.ruoyi.common.constant.PostCheckedProblemEnum;
import com.ruoyi.common.constant.PostProgrammeTemplateEnum;
import com.ruoyi.common.exception.base.BaseException;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.postCheck.mapper.PostProgrammeTemplateMapper;
import com.ruoyi.postCheck.domain.PostProgrammeTemplate;
import com.ruoyi.postCheck.service.IPostProgrammeTemplateService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

/**
 * 岗检方案模板Service业务层处理
 *
 * @author csg
 * @date 2023-12-19
 */
@Service
public class PostProgrammeTemplateServiceImpl extends ServiceImpl<PostProgrammeTemplateMapper, PostProgrammeTemplate> implements IPostProgrammeTemplateService {
    @Autowired
    private PostProgrammeTemplateMapper postProgrammeTemplateMapper;

    /**
     * 查询岗检方案模板
     *
     * @param id 岗检方案模板主键
     * @return 岗检方案模板
     */
    @Override
    public PostProgrammeTemplate selectPostProgrammeTemplateById(Long id) {
        return postProgrammeTemplateMapper.selectPostProgrammeTemplateById(id);
    }

    /**
     * 查询岗检方案模板列表
     *
     * @param postProgrammeTemplate 岗检方案模板
     * @return 岗检方案模板
     */
    @Override
    public List<PostProgrammeTemplate> selectPostProgrammeTemplateList(PostProgrammeTemplate postProgrammeTemplate) {
        return postProgrammeTemplateMapper.selectPostProgrammeTemplateList(postProgrammeTemplate);
    }

    /**
     * 新增岗检方案模板
     *
     * @param postProgrammeTemplate 岗检方案模板
     * @return 结果
     */
    @Override
    public int insertPostProgrammeTemplate(PostProgrammeTemplate postProgrammeTemplate) {
        postProgrammeTemplate.setId(Long.parseLong(IdWorker.getIdStr().substring(4)));
        postProgrammeTemplate.setStatus(null);
        postProgrammeTemplate.setCreateDefault();
        return postProgrammeTemplateMapper.insertPostProgrammeTemplate(postProgrammeTemplate);
    }

    /**
     * 修改岗检方案模板
     *
     * @param postProgrammeTemplate 岗检方案模板
     * @return 结果
     */
    @Override
    public int updatePostProgrammeTemplate(PostProgrammeTemplate postProgrammeTemplate) {

        Long id = postProgrammeTemplate.getId();
        PostProgrammeTemplate data = postProgrammeTemplateMapper.selectById(id);
        if (data == null) {
            throw new BaseException("id错误，不存在！");
        }
        Integer status = data.getStatus();
        if (status != null && status == PostProgrammeTemplateEnum.STATUS_VALID.getValue()) {
            throw new BaseException("启用中的模板，允许修改！");
        }

        postProgrammeTemplate.setUpdateDefault();
        postProgrammeTemplate.setStatus(null);
        return postProgrammeTemplateMapper.updatePostProgrammeTemplate(postProgrammeTemplate);
    }

    /**
     * 批量删除岗检方案模板
     *
     * @param ids 需要删除的岗检方案模板主键
     * @return 结果
     */
    @Override
    public int deletePostProgrammeTemplateByIds(Long[] ids) {
        return postProgrammeTemplateMapper.deletePostProgrammeTemplateByIds(ids);
    }

    /**
     * 删除岗检方案模板信息
     *
     * @param id 岗检方案模板主键
     * @return 结果
     */
    @Override
    public int deletePostProgrammeTemplateById(Long id) {
        return postProgrammeTemplateMapper.deletePostProgrammeTemplateById(id);
    }

    /**
     * 审核：岗检方案模板启用/禁用
     *
     * @param id     主键ID
     * @param status 状态（1-启动；2-作废）
     * @return
     */
    @Override
    public int approve(Long id, Integer status) {
        if (id == null || status == null) {
            throw new BaseException("参数不全，请检查！");
        }

        if (status != PostProgrammeTemplateEnum.STATUS_VALID.getValue() && status != PostProgrammeTemplateEnum.STATUS_INVALID.getValue()) {
            throw new BaseException("参数错误，请检查！【" + status + "】");
        }


        PostProgrammeTemplate param = new PostProgrammeTemplate();
        param.setUpdateDefault();
        param.setId(id);
        param.setStatus(status);

        return postProgrammeTemplateMapper.updatePostProgrammeTemplate(param);
    }
}
